import { __awaiter, __decorate } from "tslib";
import { Datasource } from '@genesislcap/foundation-comms';
import { formatDateTimestamp, formatDateTimeTimestamp } from '@genesislcap/foundation-utils';
import { customElement, FASTElement, observable, attr } from '@microsoft/fast-element';
import { logger } from '../../utils';
import { getColumns, getGridOptions } from './columns';
import { PreviewStyles as styles } from './preview.styles';
import { PreviewReportTemplate as template } from './preview.template';
const name = 'reporting-preview';
let Preview = class Preview extends FASTElement {
    constructor() {
        super(...arguments);
        this.report = {};
        /**
         * Fetch data from server and load to grid
         */
        this.fetchAndLoadToGrid = () => __awaiter(this, void 0, void 0, function* () {
            // Get the data from the selected dataserver
            // NOTE: Backend field REPORT_DATASOURCE and REPORT_DATA_SOURCE are not consistent in Events and Dataserver
            const resourceName = this.report['REPORT_DATASOURCE'] || this.report['REPORT_DATA_SOURCE'];
            const initOK = yield this.datasource.init(Object.assign({ resourceName }, this.datasourceSettings), true);
            if (!initOK) {
                logger.debug(`Genesis Datasource init failed for ${resourceName}`);
                return;
            }
            this.data = yield this.datasource.snapshotFiltered();
            // Map data from dataserver to the grid
            this.grid['gridOptions'] = Object.assign(Object.assign({}, getGridOptions()), { columnDefs: getColumns(this.report['REPORT_COLUMNS']), rowData: this.data });
        });
        /**
         * Export to csv handle
         */
        this.exportToCsv = () => {
            const params = {
                fileName: 'export.csv',
            };
            params.processCellCallback = (cellParams) => this.formatDateType(cellParams);
            this.grid.gridApi.exportDataAsCsv(Object.assign({}, params));
        };
        this.formatDateType = (cellParams) => {
            let dateFormatter;
            this.report['REPORT_COLUMNS'].forEach((column) => {
                if (column.name == cellParams.column.colId) {
                    column.type === 'DATE'
                        ? (dateFormatter = formatDateTimestamp(cellParams.value))
                        : column.type === 'DATETIME'
                            ? (dateFormatter = formatDateTimeTimestamp(cellParams.value))
                            : (dateFormatter = cellParams.value);
                }
            });
            return dateFormatter;
        };
    }
    /**
     * When report variable changes handle
     */
    reportChanged(old, newValue) {
        // Skip if empty report object
        if (old &&
            Object.keys(old).length != 0 &&
            Object.keys(newValue).length > 0 &&
            (newValue['REPORT_DATASOURCE'] || newValue['REPORT_DATA_SOURCE'])) {
            this.fetchAndLoadToGrid();
        }
    }
    /**
     * On component load
     */
    connectedCallback() {
        super.connectedCallback();
        this.fetchAndLoadToGrid();
        this.grid.addEventListener('onGridReady', () => {
            const model = {};
            this.report['REPORT_COLUMNS']
                .filter((f) => f.filter)
                .forEach((f) => {
                model[f.name] = {
                    filterType: 'text',
                    type: f.filter.conditional,
                    filter: f.filter.value,
                };
            });
            this.grid.gridApi.setFilterModel(model);
            window.addEventListener('resize', () => {
                setTimeout(() => {
                    var _a, _b;
                    (_b = (_a = this.grid) === null || _a === void 0 ? void 0 : _a.columnApi) === null || _b === void 0 ? void 0 : _b.autoSizeAllColumns();
                });
            });
            this.grid.gridApi.addEventListener('firstDataRendered', () => {
                var _a, _b;
                (_b = (_a = this.grid) === null || _a === void 0 ? void 0 : _a.columnApi) === null || _b === void 0 ? void 0 : _b.autoSizeAllColumns();
            });
        });
    }
    disconnectedCallback() {
        var _a;
        super.disconnectedCallback();
        /**
         * Attempt to clean up after ourselves.
         */
        (_a = this.datasource) === null || _a === void 0 ? void 0 : _a.destroy();
        delete this.datasource;
    }
};
__decorate([
    Datasource
], Preview.prototype, "datasource", void 0);
__decorate([
    attr
], Preview.prototype, "report", void 0);
__decorate([
    observable
], Preview.prototype, "data", void 0);
__decorate([
    observable
], Preview.prototype, "datasourceSettings", void 0);
Preview = __decorate([
    customElement({
        name,
        template,
        styles,
    })
], Preview);
export { Preview };
