import { __awaiter, __decorate, __rest } from "tslib";
import { Connect } from '@genesislcap/foundation-comms';
import { customElement, FASTElement, observable, volatile } from '@microsoft/fast-element';
import { ReportingEventListeners } from '../../routes/reporting-home/types';
import { FiltersStep, NameStep, RunStep, SelectStep } from './steps';
import { WizardStyles as styles } from './wizard.styles';
import { WizardTemplate as template } from './wizard.template';
import { WizardStep, } from './wizard.types';
const name = 'reporting-wizard';
let Wizard = class Wizard extends FASTElement {
    constructor() {
        super(...arguments);
        this.debug = false;
        this.activeTab = 0;
        this.datasourceOptions = [];
        this.datasourceFieldOptions = [];
        this.isEdit = false;
        this.fieldData = [];
        this.content = [
            {
                title: 'Set up & Datasource',
                description: "Please give your new report a name, brief description and select it's datasource to get started.",
            },
            {
                title: 'Set fields',
                description: '',
            },
            {
                title: 'Apply filters',
                description: '(Optional)',
            },
            {
                title: 'Run report',
                description: '',
            },
        ];
        // Set name and datasource step
        this.reportName = '';
        this.reportDescription = '';
        this.selectedDatasource = null;
        // Set fields step
        this.selectedFields = [];
        // Set filters step
        this.selectedFilters = [];
        // Run / preview report step
        this.previewData = [];
        this.previewGridColumns = [];
        /**
         * Get a simplified / normalized version of the fields
         * e.g. ['REPORT_ID', 'REPORT_NAME']
         * @returns Array of string
         */
        this.getNormalizedSelectedFields = () => { var _a; return ((_a = this.selectedFields) === null || _a === void 0 ? void 0 : _a.map((field) => field.displayName)) || []; };
        /**
         * Get normalized selected filters context
         * @returns Array of strings
         */
        this.getNormalizedSelectedFilters = () => { var _a; return ((_a = this.selectedFilters) === null || _a === void 0 ? void 0 : _a.map(({ displayName }) => displayName)) || []; };
        /**
         * On datasource radio group change event handler
         * @param selection Number
         */
        this.onDatasourceChange = (selection) => {
            this.selectedDatasource = selection;
        };
        /**
         * Get report context data
         * @returns Object
         */
        this.getData = () => {
            return {
                REPORT_NAME: this.reportName,
                REPORT_DESCRIPTION: this.reportDescription,
                REPORT_COLUMNS: this.selectedFields,
                REPORT_DATA_SOURCE: this.selectedDatasource,
            };
        };
        /**
         * Set the current active tab
         * @param tab Number
         */
        this.setActiveTab = (tabIndex) => {
            if (this.content[tabIndex]) {
                this.activeTab = tabIndex;
            }
        };
    }
    get isFormValid() {
        // Note: This is a very basic form validation. Would be nice to know if there are any existing form validation tools that we've used in the past and implement it here.
        switch (this.activeTab) {
            // Set up & Datasource Step
            case WizardStep.Zero:
                return this.reportName !== '' && this.selectedDatasource !== null ? false : true;
            // Set columns Step
            case WizardStep.One:
                return !this.selectedFields.length;
            // Set filters Step
            case WizardStep.Two:
                return false;
            // Run report Step
            case WizardStep.Three:
                return false;
            default:
                return false;
        }
    }
    get viewTemplate() {
        switch (this.activeTab) {
            case WizardStep.Zero:
                return NameStep;
            case WizardStep.One:
                return SelectStep;
            case WizardStep.Two:
                return FiltersStep;
            case WizardStep.Three:
                return RunStep;
            default:
                throw new Error(`Cannot get template for state ${this.activeTab}`);
        }
    }
    /**
     * When report variable changes handle
     */
    reportChanged(old, newValue) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.fieldData = (_a = this.fieldData) === null || _a === void 0 ? void 0 : _a.map((_a) => {
                var { filter } = _a, fields = __rest(_a, ["filter"]);
                return fields;
            });
            if (Object.keys(newValue).length) {
                const { REPORT_NAME, REPORT_DESCRIPTION, REPORT_DATASOURCE, REPORT_DATA_SOURCE, REPORT_COLUMNS, } = newValue;
                this.isEdit = true;
                this.selectedFields = REPORT_COLUMNS;
                this.reportName = REPORT_NAME;
                this.reportDescription = REPORT_DESCRIPTION;
                this.selectedDatasource = REPORT_DATASOURCE || REPORT_DATA_SOURCE;
            }
            else {
                this.isEdit = false;
                this.reportName = '';
                this.reportDescription = '';
                this.selectedDatasource = null;
                this.selectedFields = [];
                this.selectedFilters = [];
            }
            this.setActiveTab(0);
        });
    }
    /**
     * Set selected fields
     * @param fields array
     */
    setFields(fields) {
        this.selectedFields = fields;
    }
    /**
     * When report variable changes handle
     */
    selectedDatasourceChanged(old, newValue) {
        return __awaiter(this, void 0, void 0, function* () {
            // Reset
            this.selectedFields = [];
            this.selectedFilters = [];
            this.fieldData = [];
            // Prepare field data structure from field metadata response
            const fieldMetadata = yield this.fetchFieldsFromDatasource();
            if (this.isEdit && Object.keys(this.report).length) {
                this.selectedFields = this.report['REPORT_COLUMNS'];
                const reportDatasource = this.report['REPORT_DATASOURCE'] || this.report['REPORT_DATA_SOURCE'];
                if (reportDatasource === newValue) {
                    this.fieldData = fieldMetadata.map(({ FIELD_NAME, TYPE }) => this.selectedFields.find((field) => field.name === FIELD_NAME) || {
                        name: FIELD_NAME,
                        type: TYPE,
                        displayName: FIELD_NAME,
                        selected: false,
                    });
                }
            }
            else {
                fieldMetadata === null || fieldMetadata === void 0 ? void 0 : fieldMetadata.forEach(({ FIELD_NAME, TYPE }) => {
                    this.fieldData = [
                        ...this.fieldData,
                        {
                            name: FIELD_NAME,
                            type: TYPE,
                            displayName: FIELD_NAME,
                            selected: false,
                        },
                    ];
                });
            }
        });
    }
    /**
     * Make sure to update selected fields when field data changes
     */
    fieldDataChanged() {
        var _a, _b;
        this.selectedFields =
            ((_a = this.selectedFields) === null || _a === void 0 ? void 0 : _a.map((f) => {
                var _a;
                const matchingField = (_a = this.fieldData) === null || _a === void 0 ? void 0 : _a.find((field) => (field === null || field === void 0 ? void 0 : field.name) === (f === null || f === void 0 ? void 0 : f.name));
                return Object.assign(Object.assign({}, matchingField), { displayName: (matchingField === null || matchingField === void 0 ? void 0 : matchingField.displayName) || (f === null || f === void 0 ? void 0 : f.name), selected: matchingField ? true : false });
            })) || [];
        this.selectedFilters = ((_b = this.fieldData) === null || _b === void 0 ? void 0 : _b.filter((field) => field.filter && field.selected)) || [];
    }
    connectedCallback() {
        super.connectedCallback();
        this.getDatasources();
        this.addEventListener('setSelectedFields', (e) => {
            var _a, _b;
            this.selectedFields = (_a = e === null || e === void 0 ? void 0 : e.detail) === null || _a === void 0 ? void 0 : _a.fields;
            this.fieldData = (_b = this.fieldData) === null || _b === void 0 ? void 0 : _b.map((field) => {
                const matchingField = this.selectedFields.find((f) => (f === null || f === void 0 ? void 0 : f.name) === (field === null || field === void 0 ? void 0 : field.name));
                return Object.assign(Object.assign(Object.assign({}, field), matchingField), { displayName: (matchingField === null || matchingField === void 0 ? void 0 : matchingField.displayName) || (field === null || field === void 0 ? void 0 : field.name), selected: matchingField ? true : false });
            });
        });
        this.addEventListener('setSelectedFilters', (e) => {
            var _a;
            this.fieldData = (_a = this.fieldData) === null || _a === void 0 ? void 0 : _a.map((field) => {
                var _a, _b;
                const matchingField = (_b = (_a = e === null || e === void 0 ? void 0 : e.detail) === null || _a === void 0 ? void 0 : _a.filters) === null || _b === void 0 ? void 0 : _b.find((f) => (f === null || f === void 0 ? void 0 : f.name) === (field === null || field === void 0 ? void 0 : field.name));
                if (matchingField) {
                    matchingField.filter = Object.assign({}, matchingField.filter);
                }
                else {
                    field === null || field === void 0 ? true : delete field.filter;
                }
                return Object.assign(Object.assign({}, field), matchingField);
            });
        });
    }
    /**
     * Fetch a list of datasources
     */
    getDatasources() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.connect
                .request('ALL_REPORT_DATASOURCES', {
                REQUEST: {},
            })
                .then((response) => {
                var _a;
                (_a = response === null || response === void 0 ? void 0 : response.REPLY) === null || _a === void 0 ? void 0 : _a.forEach(({ DATASOURCE_NAME, RECORD_ID }) => {
                    this.datasourceOptions = [
                        ...this.datasourceOptions,
                        {
                            id: RECORD_ID,
                            name: DATASOURCE_NAME,
                        },
                    ];
                });
            })
                .catch((err) => {
                this.$emit(ReportingEventListeners.notify, {
                    type: 'warning',
                    title: 'Error:',
                    message: `There was an error fetching data sources:
          ${(err === null || err === void 0 ? void 0 : err.ERROR) ? err.ERROR : JSON.stringify(err)}
        `,
                });
            });
        });
    }
    /**
     * Fetch data from the provided datasource
     */
    fetchFieldsFromDatasource() {
        return (this.selectedDatasource &&
            this.connect
                .getMetadata(this.selectedDatasource)
                .then((response) => {
                if (response.ERROR) {
                    this.$emit(ReportingEventListeners.notify, {
                        type: 'warning',
                        title: 'Error:',
                        message: `The application encountered an error while trying to fetch fields
              ${this.selectedDatasource && `for data source "${this.selectedDatasource}"`}: 
              ${response.ERROR} 
            `,
                    });
                }
                const fields = (response === null || response === void 0 ? void 0 : response.FIELD) || (response === null || response === void 0 ? void 0 : response.REPLY_FIELD);
                return fields === null || fields === void 0 ? void 0 : fields.map(({ NAME, TYPE }) => ({ FIELD_NAME: NAME, TYPE }));
            })
                .catch((err) => {
                this.$emit(ReportingEventListeners.notify, {
                    type: 'warning',
                    title: 'Error:',
                    message: (err === null || err === void 0 ? void 0 : err.ERROR) ? err.ERROR : JSON.stringify(err),
                });
            }));
    }
};
__decorate([
    Connect
], Wizard.prototype, "connect", void 0);
__decorate([
    observable
], Wizard.prototype, "debug", void 0);
__decorate([
    observable
], Wizard.prototype, "activeTab", void 0);
__decorate([
    observable
], Wizard.prototype, "datasourceOptions", void 0);
__decorate([
    observable
], Wizard.prototype, "datasourceFieldOptions", void 0);
__decorate([
    observable
], Wizard.prototype, "isEdit", void 0);
__decorate([
    observable
], Wizard.prototype, "fieldData", void 0);
__decorate([
    observable
], Wizard.prototype, "report", void 0);
__decorate([
    observable
], Wizard.prototype, "saveReport", void 0);
__decorate([
    observable
], Wizard.prototype, "content", void 0);
__decorate([
    observable
], Wizard.prototype, "datasourceSettings", void 0);
__decorate([
    observable
], Wizard.prototype, "reportName", void 0);
__decorate([
    observable
], Wizard.prototype, "reportDescription", void 0);
__decorate([
    observable
], Wizard.prototype, "selectedDatasource", void 0);
__decorate([
    observable
], Wizard.prototype, "selectedFields", void 0);
__decorate([
    observable
], Wizard.prototype, "selectedFilters", void 0);
__decorate([
    observable
], Wizard.prototype, "previewData", void 0);
__decorate([
    observable
], Wizard.prototype, "previewGridColumns", void 0);
__decorate([
    volatile
], Wizard.prototype, "isFormValid", null);
__decorate([
    volatile
], Wizard.prototype, "viewTemplate", null);
Wizard = __decorate([
    customElement({
        name,
        template,
        styles,
    })
], Wizard);
export { Wizard };
